      subroutine datain(par)
************************************************************************
*                                                                      *
*     Read in the data from the input file.                            *
*     Version of Jan 18, 1999.                                         *
*                                                                      *
************************************************************************
$include grids.h
$include apeno.h
$include datacom.h
      double precision par(*)
      common /extras/ eps, h1, hmin, alsq, neqn
************************************************************************
*                                                                      *
*     Read the x-grid information and convert to useful forms.         *
*     Note that the x values must be given in ascending order.         *
*                                                                      *
************************************************************************
      read(*,*) nxgrd
      if (nxgrd.gt.mxgrd) then
         write(*,'(a,i5)') '[ERROR] Number of x grids > ', mxgrd
         stop
      end if
      read(*,*) (x0(i),i=1,nxgrd)
      write(*,'(a,i4,a)') '# Read ', nxgrd, ' x-grid points'
      write(*,'(5e12.4)') (x0(i),i=1,nxgrd)
*
      call apeinit
      write(*,'(a)') '# Initialised array of splitting functions'
************************************************************************
*                                                                      *
*     Read the q-grid information and convert to useful forms.         *
*     Note that the q values must be given in ascending order.         *
*                                                                      *
************************************************************************
      read(*,*) nqgrd
      if (nqgrd+4.gt.mqgrd) then
         write(*,'(a,i5)') '[ERROR] Number of q grids > ', mqgrd-4
         stop
      end if
      read(*,*) (q2(i),i=1,nqgrd)
      read(*,*) kfin
      q2sp=q2(kfin)
      call thrsh(alsq5,alsq4,alsq3)
      kfin=iq2sch(q2sp)
      write(*,'(a,i4,a)') '# Read ', nqgrd, ' t-grid points'
      write(*,'(a,e12.4)') '# Special point: q2= ', q2(kfin)
************************************************************************
*     Some constants needed for solution of odes: used in FUNC         *
************************************************************************
      neqn=meqn*nxgrd
      read(*,*) eps
************************************************************************
*                                                                      *
*     Read the encoding of the data and nuclear correction factor.     *
*     The key corresponding to an experiment is illegal if it is       *
*     not equal to "kprot" or "kdeut". Data for illegal keys will      *
*     not be used in the fits.                                         *
*                                                                      *
************************************************************************
      read(*,*) nexp
      if ((nexp.le.0).or.(nexp.ge.mexp)) then
         write(*,'(a,i5)') '[ERROR] Number of experiments > ', mexp
         stop
      end if
      read(*,*) (key(i),i=1,nexp)
************************************************************************
*                                                                      *
*     Read the x-grid for the data (acending order again).             *
*                                                                      *
************************************************************************
      read(*,*) ndx
      if (ndx.gt.mxgrd) then
         write(*,'(a,i5)') '[ERROR] Number of x grids (data) > ', mxgrd
         stop
      end if
      read(*,*) (xd(i),i=1,ndx)
      write(*,'(a,i4,a)') '# Read ', ndx, ' x-grid points for data'
      call jaldi
************************************************************************
*                                                                      *
*     Initialise the arrays which hold the error bars.                 *
*                                                                      *
************************************************************************
      read(*,*) ndata
      if (ndata.gt.mdata) then
         write(*,'(a,i5)') '[ERROR] Number of data points > ', mdata
         stop
      end if
*
      do 40 i=1,nqgrd
         do 40 j=1,nexp
            do 40 k=1,ndx
 40            indx(k,j,i)=0
************************************************************************
*                                                                      *
*     Read the data and store it into appropriate arrays               *
*     (only if the key value is legal).                                *
*                                                                      *
************************************************************************
      ndt=0
      do 45 i=1,ndata
         read(*,*) iexp, kode, ix, qsq, dt, er
         if ((key(iexp).eq.kprot).or.(key(iexp).eq.kdeut).or.
     1       (key(iexp).eq.kneut)) then
            iq=iq2sch(qsq)
            ndt=ndt+1
            if (indx(ix,iexp,iq).gt.0) then
               write(*,*) '[ERROR] Overwriting data'
               stop
            end if
            indx(ix,iexp,iq)=ndt
            d(ndt)=dt
            sg(ndt)=er*er
         end if
 45      continue
      write(*,'(a,i4,a)') '# Read ', ndata, ' pieces of data'
************************************************************************
*                                                                      *
*     Convert qsq grid into grid of 2 log(q/lambda).                   *
*                                                                      *
************************************************************************
      do 50 iq=0,nqgrd
         if (neff(iq).eq.3) then
            q2(iq)=log(q2(iq)/alsq3)
         else if (neff(iq).eq.4) then
            q2(iq)=log(q2(iq)/alsq4)
         else
            q2(iq)=log(q2(iq)/alsq5)
         end if
 50      continue
************************************************************************
*                                                                      *
*     Convert the asymmetries to the polarised structure function.     *
*                                                                      *
************************************************************************
      call unpol
      write(*,'(a,i4,a)') '# Scaled ', ndt, ' pieces of data'
*
      return
      end
      subroutine jaldi
************************************************************************
*                                                                      *
*     Some arrays to speed up the integration routine.                 *
*                                                                      *
************************************************************************
$include grids.h
$include gss32.h
$include speed.h
$include dsped.h
$include datacom.h
************************************************************************
*     Log(1-x) precomputed for the right hand side of the apes.        *
************************************************************************
      do 10 i=1,nxgrd
 10      xl(i)=log(1-x0(i))
************************************************************************
*     Log(1-x) precomputed for the coefficient functions.              *
************************************************************************
      do 15 i=1,ndx
 15      xld(i)=log(1-xd(i))
************************************************************************
*     Grid seperation for use in the spline interpolation.             *
************************************************************************
      do 20 i=1,nxgrd-1
 20      hx(i)=x0(i+1)-x0(i)
************************************************************************
*     The index for computing the spline interpolation.                *
************************************************************************
      do 30 i=1,nxgrd
         xm=0.5*(1+x0(i))
         dx=0.5*(1-x0(i))
         klo(i,0)=mkindx(x0(i))
         do 30 j=1,mgaus
            z=xm+dx*xg(j)
            klo(i,j)=mkindx(x0(i)/z)
            z=xm-dx*xg(j)
 30         klo(i,mgaus+j)=mkindx(x0(i)/z)
************************************************************************
*     The spline indices for the data grid.                            *
************************************************************************
      do 40 i=1,ndx
         xm=0.5*(1+xd(i))
         dx=0.5*(1-xd(i))
         klod(i,0)=mkindx(xd(i))
         do 40 j=1,mgaus
            z=xm+dx*xg(j)
            klod(i,j)=mkindx(xd(i)/z)
            z=xm-dx*xg(j)
 40         klod(i,mgaus+j)=mkindx(xd(i)/z)
*
      return
      end
      function mkindx(x)
************************************************************************
*                                                                      *
*     Compute the search index of the input in the grid.               *
*                                                                      *
************************************************************************
$include grids.h
*
      klo=1
      khi=nxgrd
1     if (khi-klo.gt.1) then
         k=(khi+klo)/2
         if (x0(k).gt.x) then
            khi=k
         else
            klo=k
         end if
         go to 1
      end if
      mkindx=klo
*
      return
      end
      function iq2sch(q2m)
************************************************************************
*                                                                      *
*     Compute the search index of the input in the grid.               *
*                                                                      *
************************************************************************
$include grids.h
*
      klo=0
      khi=nqgrd
1     if (khi-klo.gt.1) then
         k=(khi+klo)/2
         if (q2(k).gt.q2m) then
            khi=k
         else
            klo=k
         end if
         go to 1
      end if
      iq2sch=klo
*
      return
      end
      subroutine apeinit
************************************************************************
*                                                                      *
*  Precompute combinations of NLO splitting functions for use in the   *
*  Altarelli-Parisi evolution routines for polarised densities.        *
*                                                                      *
*  Version of Jan 20, 2000.                                            *
*                                                                      *
************************************************************************
$include qcd.h
$include grids.h
$include gss32.h
$include apesf.h
************************************************************************
*     The main control loop.                                           *
************************************************************************
      do 10 ix=1,nxgrd-1
         z0=0.5*(1+x0(ix))
         dz=0.5*(1-x0(ix))
*
         do 11 ig=1,mgaus
************************************************************************
*           One of the Gauss-Legendre points.                          *
************************************************************************
            z=z0+dz*xg(ig)
            zm1=1-z
            z2p1=1+z*z
            z2m1=1-2*z
            zln=log(z)
            zlnspl=zln*(11/3.+zln)
            zl1=log(1-z)
            s22=zln*(zln-4*log(1+z))-2*zeta2-4*dilog(-z)
************************************************************************
*           The splitting functions at LO.                             *
************************************************************************
            pqq=cf*z2p1/zm1
            pqg=-2*z2m1
            pggr=cg02*z2m1
            pggs=cg02/zm1
            pgq=cf*(2-z)
*
            spl1(1,ig,ix)=pqq
            spl1(2,ig,ix)=pqg
            spl1(3,ig,ix)=pgq
            spl1(4,ig,ix)=pggr
            spl1(5,ig,ix)=pggs
************************************************************************
*           The splitting functions at NLO.                            *
************************************************************************
            pqqa=cf*2*((11*z-1)/3+(1+z-2/zm1)*zln)/3.
            pqqb=cf*(cf*(9*(z-1)-zln*(2+4*z+3/zm1+0.5*zln*(1+z))
     1                     -s22*(2/(1+z)-zm1)+2*zln*zl1*(1+z-2/zm1))
     2               +ca*(26*zm1/3+2*(1+z)*zln+s22*(1/(1+z)-zm1/2)
     3                     +zlnspl/zm1-0.5*(cpaf1+zlnspl)*(1+z)))
            pqqc=cf*(cf*(z-1+zln*(2-3/zm1-0.5*zln*(1+z))
     1                     +s22*(2/(1+z)-zm1)+2*zln*zl1*(1+z-2/zm1))
     2               +ca*(14*zm1/3-s22*(1/(1+z)-zm1/2.)
     3                     +zlnspl/zm1-0.5*(cpaf1+zlnspl)*(1+z)))
            psna=cf*(4*(4+z)/9-zln*(4*(1-5*z+1/zm1)/3+2*(1+z)*zln))
            psnb=-20/zm1/9.
            psnc=ca*cpaf1/zm1
            pqgr=cf*(-22+27*z-9*zln+4*zm1*zl1*(2-zl1)
     1                       +z2m1*(4*zl1*zln-zln*zln+4*zeta2))
     2          +ca*(2*(12-11*z)-4*zm1*zl1*(2-zl1)+2*(1+8*z)*zln
     3                  -2*zeta2*z2m1+(1+2*z)*(s22-3*zln*zln))
            pqgs=2*(cf-ca)*zl1*zl1
            pgqa=-4*cf*((4+z)/3+zm1*zl1)/3.
            pgqb=cf*(cf*(-0.5*(17-8*z)-0.5*zln*(4-z-zln*(2-z))
     1                         +zm1*zl1*(1-zl1))
     2              +ca*((41+35*z)/9+zln*(4-13*z+1.5*zln*(2+z))
     3                       -0.5*(2+z)*s22-(2-z)*(zeta2+2*zln*zl1)
     4                            -zm1*zl1*(1/3.-zl1)))
            pgqc=-4*cf*zl1/3
            pgqd=-cf*zl1*(cf*(3+zl1)-ca*(11/3.-zl1))
            pgga=-2*cf*(5*zm1+zln*(5-z+(1+z)*zln))
     1           -4*ca*((14-19*z)/3.+(1+z)*zln)/3.
            pggb=ca*ca*(-19*zm1/2+zln*((29-67*z)/3+4*(1+z)*zln)
     1                 -s22*(1/(1+z)+2*z+1)+z2m1*cpaf1
     2                 +zln*(zln-4*zl1)*(1/zm1+z2m1))
*
            spl2(1,ig,ix) =pqqa
            spl2(2,ig,ix) =pqqb
            spl2(3,ig,ix) =pqqc
            spl2(4,ig,ix) =psna
            spl2(5,ig,ix) =psnb
            spl2(6,ig,ix) =psnc
            spl2(7,ig,ix) =pqgr
            spl2(8,ig,ix) =pqgs
            spl2(9,ig,ix) =pgqa
            spl2(10,ig,ix)=pgqb
            spl2(11,ig,ix)=pgqc
            spl2(12,ig,ix)=pgqd
            spl2(13,ig,ix)=pgga
            spl2(14,ig,ix)=pggb
************************************************************************
*           Another of the Gauss-Legendre points.                      *
************************************************************************
            z=z0-dz*xg(ig)
            zm1=1-z
            z2p1=1+z*z
            z2m1=1-2*z
            zln=log(z)
            zlnspl=zln*(11/3.+zln)
            zl1=log(1-z)
            s22=zln*(zln-4*log(1+z))-2*zeta2-4*dilog(-z)
************************************************************************
*           The splitting functions at LO.                             *
************************************************************************
            pqq=cf*z2p1/zm1
            pqg=-2*z2m1
            pggr=cg02*z2m1
            pggs=cg02/zm1
            pgq=cf*(2-z)
*
            spl1(1,mgaus+ig,ix)=pqq
            spl1(2,mgaus+ig,ix)=pqg
            spl1(3,mgaus+ig,ix)=pgq
            spl1(4,mgaus+ig,ix)=pggr
            spl1(5,mgaus+ig,ix)=pggs
************************************************************************
*           The splitting functions at NLO.                            *
************************************************************************
            pqqa=cf*2*((11*z-1)/3+(1+z-2/zm1)*zln)/3.
            pqqb=cf*(cf*(9*(z-1)-zln*(2+4*z+3/zm1+0.5*zln*(1+z))
     1                     -s22*(2/(1+z)-zm1)+2*zln*zl1*(1+z-2/zm1))
     2               +ca*(26*zm1/3+2*(1+z)*zln+s22*(1/(1+z)-zm1/2)
     3                     +zlnspl/zm1-0.5*(cpaf1+zlnspl)*(1+z)))
            pqqc=cf*(cf*(z-1+zln*(2-3/zm1-0.5*zln*(1+z))
     1                     +s22*(2/(1+z)-zm1)+2*zln*zl1*(1+z-2/zm1))
     2               +ca*(14*zm1/3-s22*(1/(1+z)-zm1/2.)
     3                     +zlnspl/zm1-0.5*(cpaf1+zlnspl)*(1+z)))
            psna=cf*(4*(4+z)/9-zln*(4*(1-5*z+1/zm1)/3+2*(1+z)*zln))
            psnb=-20/zm1/9.
            psnc=ca*cpaf1/zm1
            pqgr=cf*(-22+27*z-9*zln+4*zm1*zl1*(2-zl1)
     1                       +z2m1*(4*zl1*zln-zln*zln+4*zeta2))
     2          +ca*(2*(12-11*z)-4*zm1*zl1*(2-zl1)+2*(1+8*z)*zln
     3                  -2*zeta2*z2m1+(1+2*z)*(s22-3*zln*zln))
            pqgs=2*(cf-ca)*zl1*zl1
            pgqa=-4*cf*((4+z)/3+zm1*zl1)/3.
            pgqb=cf*(cf*(-0.5*(17-8*z)-0.5*zln*(4-z-zln*(2-z))
     1                         +zm1*zl1*(1-zl1))
     2              +ca*((41+35*z)/9+zln*(4-13*z+1.5*zln*(2+z))
     3                       -0.5*(2+z)*s22-(2-z)*(zeta2+2*zln*zl1)
     4                            -zm1*zl1*(1/3.-zl1)))
            pgqc=-4*cf*zl1/3
            pgqd=-cf*zl1*(cf*(3+zl1)-ca*(11/3.-zl1))
            pgga=-2*cf*(5*zm1+zln*(5-z+(1+z)*zln))
     1           -4*ca*((14-19*z)/3.+(1+z)*zln)/3.
            pggb=ca*ca*(-19*zm1/2+zln*((29-67*z)/3+4*(1+z)*zln)
     1                 -s22*(1/(1+z)+2*z+1)+z2m1*cpaf1
     2                 +zln*(zln-4*zl1)*(1/zm1+z2m1))
*
            spl2(1,mgaus+ig,ix) =pqqa
            spl2(2,mgaus+ig,ix) =pqqb
            spl2(3,mgaus+ig,ix) =pqqc
            spl2(4,mgaus+ig,ix) =psna
            spl2(5,mgaus+ig,ix) =psnb
            spl2(6,mgaus+ig,ix) =psnc
            spl2(7,mgaus+ig,ix) =pqgr
            spl2(8,mgaus+ig,ix) =pqgs
            spl2(9,mgaus+ig,ix) =pgqa
            spl2(10,mgaus+ig,ix)=pgqb
            spl2(11,mgaus+ig,ix)=pgqc
            spl2(12,mgaus+ig,ix)=pgqd
            spl2(13,mgaus+ig,ix)=pgga
            spl2(14,mgaus+ig,ix)=pggb
 11         continue
 10       continue
*
      return
      end
